//
// Attribute.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * An 'abstract' base class used to represent any attribute record in a Java
 * <tt>.class</tt> file.
 *
 * @version 1.1, 01 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John Leach</a>
 * @see AttributeFactory
 * @see SourceFileAttribute
 * @see ConstantValueAttribute
 * @see CodeAttribute
 * @see ExceptionsAttribute
 * @see LineNumberTableAttribute
 * @see LocalVariableTableAttribute
**/

public class Attribute {

 /** The Unknown attribute tag. */
 final static int UNKNOWN = 0;

 /** The SourceFile attribute tag. */
 final static int SOURCE_FILE = 1;

 /** The ConstantValue attribute tag. */
 final static int CONSTANT_VALUE = 2;

 /** The Code attribute tag. */
 final static int CODE = 3;

 /** The Exceptions attribute tag. */
 final static int EXCEPTIONS = 4;

 /** The LineNumberTable attribute tag. */
 final static int LINE_NUMBER_TABLE = 5;

 /** The LocalVaraibleTable attribute tag. */
 final static int LOCAL_VARIABLE_TABLE = 6;

 /** The attribute type. */
 int type;

 /** The attribute size. */
 int size;

 /**
  * Allows sub-classing of the constructor.
 **/

 protected Attribute() { }

 /**
  * Gets the attribute as a string, does nothing, must be over-ridden.
  * @exception ClassException	    if the record cannot be decoded
 **/

 public String stringize() throws ClassException {
  throw new ClassException("Internal Error: Attribute.stringize() not over-ridden");
  }
 }
