//
// Constant.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * An 'abstract' base class used to represent any CONSTANT_xxx record in a Java
 * <tt>.class</tt> file.
 *
 * @version 1.1, 01 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John Leach</a>
 * @see ConstantFactory
 * @see ConstantUtf8
 * @see ConstantUnicode
 * @see ConstantInteger
 * @see ConstantFloat
 * @see ConstantLong
 * @see ConstantDouble
 * @see ConstantClass
 * @see ConstantString
 * @see ConstantFieldRef
 * @see ConstantMethodRef
 * @see ConstantInterfaceMethodRef
 * @see ConstantNameAndType
**/

public class Constant {

 /** The Constant Pool. */
 static Constant[] constantPool = null;

 /** The CONSTANT_Utf8 record tag. */
 final static int UTF8 = 1;

 /** The CONSTANT_Unicode record tag. */
 final static int UNICODE = 2;

 /** The CONSTANT_Integer record tag. */
 final static int INTEGER = 3;

 /** The CONSTANT_Float record tag. */
 final static int FLOAT = 4;

 /** The CONSTANT_Long record tag. */
 final static int LONG = 5;

 /** The CONSTANT_Double record tag. */
 final static int DOUBLE = 6;

 /** The CONSTANT_Class record tag. */
 final static int CLASS = 7;

 /** The CONSTANT_String record tag. */
 final static int STRING = 8;

 /** The CONSTANT_Fieldref record tag. */
 final static int FIELD_REF = 9;

 /** The CONSTANT_Methodref record tag. */
 final static int METHOD_REF = 10;

 /** The CONSTANT_InterfaceMethodref record tag. */
 final static int INTERFACE_METHOD_REF = 11;

 /** The CONSTANT_NameAndType record tag. */
 final static int NAME_AND_TYPE = 12;

 /** The record type. */
 int type;

 /** The record size. */
 int size;

 /**
  * Allows sub-classing of the constructor.
 **/

 protected Constant() { }

 /**
  * Sets the constant pool.
  * @param pool    the constant pool
 **/

 static void setPool(Constant[] pool) { constantPool = pool; }

 /**
  * Gets the CONSTANT_xxx as a string, does nothing, must be over-ridden.
  * @exception ClassException	    if the record cannot be decoded
 **/

 public String stringize() throws ClassException {
  throw new ClassException("Internal Error: Constant.stringize() not over-ridden");
  }
 }
