//
// ConstantClass.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * A container for a CONSTANT_Class record in a Java <tt>.class</tt> file.
 *
 * @version 1.1, 01 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John Leach</a>
 * @see ConstantFactory
 * @see ConstantUtf8
 * @see ConstantUnicode
 * @see ConstantInteger
 * @see ConstantFloat
 * @see ConstantLong
 * @see ConstantDouble
 * @see ConstantString
 * @see ConstantFieldRef
 * @see ConstantMethodRef
 * @see ConstantInterfaceMethodRef
 * @see ConstantNameAndType
**/

public final class ConstantClass extends Constant {

 /** The class name index. */
 int className;

 /** The exception error message. */
 private static final String error = "Class record corrupt: ";

 /** The class string. */
 private String classString;

 /** The class as a signature. */
 private Signature sig;

 /**
  * Creates a CONSTANT_Class record from a byte array.
  * @param buffer     the byte array
  * @param index      the offset in the input buffer
  * @exception ClassException	    if the record cannot be decoded
 **/

 public ConstantClass(byte[] buffer, int index) throws ClassException {
  try {
   className = ValueDecoder.readU2(buffer, index);

   // update the size and type

   size = 2 + 1;
   type = Constant.CLASS;
   }
  catch(ArrayIndexOutOfBoundsException e) {
   throw new ClassException(error + index);
   }
  }

 /**
  * Gets the CONSTANT_Class as a signature.
  * @exception ClassException	    if the record cannot be decoded
 **/

 public Signature getSignature() throws ClassException {
  if(classString == null) {
   Constant type;

   try {
    type = constantPool[className];
    }
   catch(NullPointerException e) {
    throw new ClassException(error + "[no entry]");
    }
   catch(ArrayIndexOutOfBoundsException e) {
    throw new ClassException(error + "[no entry]");
    }

   if(type.type == Constant.UTF8) classString = type.stringize();
   else throw new ClassException(error + "[wrong type]");
   }

  if(sig == null) {
   String signature = classString;
   if(classString.charAt(0) != '[') signature = "L" + classString + ";";
   sig = new Signature(signature);
   }

  return sig;
  }

 /**
  * Gets the CONSTANT_Class as a string.
  * @exception ClassException	    if the record cannot be decoded
 **/

 public String stringize() throws ClassException {
  if(classString == null) {
   Constant type;

   try {
    type = constantPool[className];
    }
   catch(NullPointerException e) {
    throw new ClassException(error + "[no entry]");
    }
   catch(ArrayIndexOutOfBoundsException e) {
    throw new ClassException(error + "[no entry]");
    }

   if(type.type == Constant.UTF8) classString = type.stringize();
   else throw new ClassException(error + "[wrong type]");
   }

  return classString;
  }
 }
