//
// ConstantFactory.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * A factory for any CONSTANT_xxx record in a Java <tt>.class</tt> file.
 *
 * @version 1.0, 01 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John Leach</a>
 * @see Constant
 * @see ConstantUtf8
 * @see ConstantUnicode
 * @see ConstantInteger
 * @see ConstantFloat
 * @see ConstantLong
 * @see ConstantDouble
 * @see ConstantClass
 * @see ConstantString
 * @see ConstantFieldRef
 * @see ConstantMethodRef
 * @see ConstantInterfaceMethodRef
 * @see ConstantNameAndType
**/

public final class ConstantFactory {

 /** The buffer offset */
 public int offset;

 /**
  * Creates a series of CONSTANT_xxx records from a byte array.
  * @param pool       the constant pool
  * @param buffer     the byte array
  * @param index      the offset in the input buffer
  * @exception ClassException	    if the record cannot be decoded
 **/

 public ConstantFactory(Constant[] pool, byte[] buffer, int index) throws ClassException {
  Constant.setPool(pool);

  for(int i = 1; i < pool.length; i++) {
   int type = (buffer[index] & 0xFF);
   ++index;

   Constant record;

   switch(type) {
    case Constant.UTF8: 		record = new ConstantUtf8(buffer, index);		break;
    case Constant.UNICODE:		record = new ConstantUnicode(buffer, index);		break;
    case Constant.INTEGER:		record = new ConstantInteger(buffer, index);		break;
    case Constant.FLOAT:		record = new ConstantFloat(buffer, index);		break;
    case Constant.LONG: 		record = new ConstantLong(buffer, index);		break;
    case Constant.DOUBLE:		record = new ConstantDouble(buffer, index);		break;
    case Constant.CLASS:		record = new ConstantClass(buffer, index);		break;
    case Constant.STRING:		record = new ConstantString(buffer, index);		break;
    case Constant.FIELD_REF:		record = new ConstantFieldRef(buffer, index);		break;
    case Constant.METHOD_REF:		record = new ConstantMethodRef(buffer, index);		break;
    case Constant.INTERFACE_METHOD_REF: record = new ConstantInterfaceMethodRef(buffer, index); break;
    case Constant.NAME_AND_TYPE:	record = new ConstantNameAndType(buffer, index);	break;

    default: throw new ClassException("Internal error: Unknown CONSTANT record type: " + (index - 1));
    }

   pool[i] = record;
   index += record.size - 1;

   // CONSTANT_long and CONSTANT_double take up two slots

   if(record.type == Constant.LONG || record.type == Constant.DOUBLE) ++i;
   }

  // update the buffer offset

  offset = index;
  }
 }
