//
// ConstantFloat.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * A container for a CONSTANT_Float record in a Java <tt>.class</tt> file.
 *
 * @version 1.1, 01 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John Leach</a>
 * @see ConstantFactory
 * @see ConstantUtf8
 * @see ConstantUnicode
 * @see ConstantInteger
 * @see ConstantLong
 * @see ConstantDouble
 * @see ConstantClass
 * @see ConstantString
 * @see ConstantFieldRef
 * @see ConstantMethodRef
 * @see ConstantInterfaceMethodRef
 * @see ConstantNameAndType
**/

public final class ConstantFloat extends Constant {

 /** The record as a float value. */
 float floatValue;

 /** The exception error message. */
 private static final String error = "Float record corrupt: ";

 /** The record as a String. */
 private String floatName;

 /**
  * Creates a CONSTANT_Float record from a byte array.
  * @param buffer     the byte array
  * @param index      the offset in the input buffer
  * @exception ClassException	    if the record cannot be decoded
 **/

 public ConstantFloat(byte[] buffer, int index) throws ClassException {
  try {
   int value = ValueDecoder.readU4(buffer, index);
   floatValue = Float.intBitsToFloat(value);
   floatName = Float.toString(floatValue);

   // update the size and type

   size = 4 + 1;
   type = Constant.FLOAT;
   }
  catch(ArrayIndexOutOfBoundsException e) {
   throw new ClassException(error + index);
   }
  }

 /**
  * Gets the CONSTANT_Float as a string.
 **/

 public String stringize() {
  return floatName;
  }
 }
