//
// ConstantNameAndType.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * A container for a CONSTANT_NameAndType record in a Java <tt>.class</tt> 
 * file.
 *
 * @version 1.1, 01 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John Leach</a>
 * @see ConstantFactory
 * @see ConstantUtf8
 * @see ConstantUnicode
 * @see ConstantInteger
 * @see ConstantFloat
 * @see ConstantLong
 * @see ConstantDouble
 * @see ConstantClass
 * @see ConstantString
 * @see ConstantFieldRef
 * @see ConstantMethodRef
 * @see ConstantInterfaceMethodRef
**/

public final class ConstantNameAndType extends Constant {

 /** The name index. */
 int nameIndex;

 /** The signature index. */
 int signatureIndex;

 /** The exception error message. */
 private static final String error = "NameAndType record corrupt: ";

 /** The name as a String. */
 private String name;

 /** The name and type as a String. */
 private String nameAndType;

 /** The type signature. */
 private Signature sig;

 /**
  * Creates a CONSTANT_NameAndType record from a byte array.
  * @param buffer     the byte array
  * @param index      the offset in the input buffer
  * @exception ClassException	    if the record cannot be decoded
 **/

 public ConstantNameAndType(byte[] buffer, int index) throws ClassException {
  try {
   nameIndex = ValueDecoder.readU2(buffer, index);
   index += 2;

   signatureIndex = ValueDecoder.readU2(buffer, index);

   // update the size and type

   size = 4 + 1;
   type = Constant.NAME_AND_TYPE;
   }
  catch(ArrayIndexOutOfBoundsException e) {
   throw new ClassException(error + index);
   }
  }

 /**
  * Gets the type signature.
  * @exception ClassException	    if the record cannot be decoded
 **/

 public Signature getSignature() throws ClassException {
  if(sig == null) {
   Constant type;
   try {
    type = constantPool[signatureIndex];
    }
   catch(NullPointerException e) {
    throw new ClassException(error + "[no entry for signature]");
    }
   catch(ArrayIndexOutOfBoundsException e) {
    throw new ClassException(error + "[no entry for signature]");
    }

   if(type.type == Constant.UTF8) sig = new Signature(type.stringize());
   else throw new ClassException(error + "[wrong type for signature]");
   }
  return sig;
  }

 /**
  * Gets the CONSTANT_NameAndType name as a string.
  * @exception ClassException	    if the record cannot be decoded
 **/

 public String getName() throws ClassException {
  if(name == null) {
   Constant type;

   try {
    type = constantPool[nameIndex];
    }
   catch(NullPointerException e) {
    throw new ClassException(error + "[no entry for name]");
    }
   catch(ArrayIndexOutOfBoundsException e) {
    throw new ClassException(error + "[no entry for name]");
    }

   if(type.type == Constant.UTF8) name = type.stringize();
   else throw new ClassException(error + "[wrong type for name]");
   }

  return name;
  }

 /**
  * Gets the CONSTANT_NameAndType as a string.
  * @exception ClassException	    if the record cannot be decoded
 **/

 public String stringize() throws ClassException {
  if(nameAndType == null) {
   Constant type;

   if(name == null) getName();
   if(sig == null) getSignature();

   nameAndType = sig.typeToString() + " " + name + sig.paramsToString();
   }

  return nameAndType;
  }
 }
