//
// ConstantString.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * A container for a CONSTANT_String record in a Java <tt>.class</tt> file.
 *
 * @version 1.1, 01 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John leach</a>
 * @see ConstantFactory
 * @see ConstantUtf8
 * @see ConstantUnicode
 * @see ConstantInteger
 * @see ConstantFloat
 * @see ConstantLong
 * @see ConstantDouble
 * @see ConstantClass
 * @see ConstantFieldRef
 * @see ConstantMethodRef
 * @see ConstantInterfaceMethodRef
 * @see ConstantNameAndType
**/

public final class ConstantString extends Constant {

 /** The string index. */
 int stringIndex;

 /** The exception error message. */
 private static final String error = "String record corrupt: ";

 /** The string as a String. */
 private String string;

 /**
  * Creates a CONSTANT_String record from a byte array.
  * @param buffer     the byte array
  * @param index      the offset in the input buffer
  * @exception ClassException	    if the record cannot be decoded
 **/

 public ConstantString(byte[] buffer, int index) throws ClassException {
  try {
   stringIndex = ValueDecoder.readU2(buffer, index);

   // update the size and type

   size = 2 + 1;
   type = Constant.STRING;
   }
  catch(ArrayIndexOutOfBoundsException e) {
   throw new ClassException(error + index);
   }
  }

 /**
  * Gets the CONSTANT_String as a string.
  * @exception ClassException	    if the record cannot be decoded
 **/

 public String stringize() throws ClassException {
  if(string == null) {
   Constant type;

   try {
    type = constantPool[stringIndex];
    }
   catch(NullPointerException e) {
    throw new ClassException(error + "[no entry]");
    }
   catch(ArrayIndexOutOfBoundsException e) {
    throw new ClassException(error + "[no entry]");
    }

   if(type.type == Constant.UTF8) string = "\"" + type.stringize() + "\"";
   else throw new ClassException(error + "[wrong type]");
   }

  return string;
  }
 }
