//
// ConstantUnicode.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * A container for a CONSTANT_Unicode record in a Java <tt>.class</tt> file.
 *
 * @version 1.1, 01 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John Leach</a>
 * @see ConstantFactory
 * @see ConstantUtf8
 * @see ConstantInteger
 * @see ConstantFloat
 * @see ConstantLong
 * @see ConstantDouble
 * @see ConstantClass
 * @see ConstantString
 * @see ConstantFieldRef
 * @see ConstantMethodRef
 * @see ConstantInterfaceMethodRef
 * @see ConstantNameAndType
**/

public final class ConstantUnicode extends Constant {

 /** The record as a String. */
 String unicode;

 /** The exception error message. */
 private static final String error = "Unicode record corrupt: ";

 /**
  * Creates a CONSTANT_Unicode record from a byte array.
  * @param buffer     the byte array
  * @param index      the offset in the input buffer
  * @exception ClassException	    if the record cannot be decoded
 **/

 public ConstantUnicode(byte[] buffer, int index) throws ClassException {
  try {
   size = ValueDecoder.readU2(buffer, index);
   index += 2;

   if((size & 0x0001) == 0) {
    char unicodeChars[] = new char[size];
    int length = size;
    int unicodeLength = 0;

    while(length > 0) {
     unicodeChars[unicodeLength] = (char)ValueDecoder.readU2(buffer, index);
     ++unicodeLength;
     index += 2;
     length -= 2;
     }

    unicode = new String(unicodeChars);
    }
   else throw new ClassException(error + index);

   // update the size and type

   size += (2 + 1);
   type = Constant.UNICODE;
   }
  catch(ArrayIndexOutOfBoundsException e) {
   throw new ClassException(error + index);
   }
  }

 /**
  * Gets the CONSTANT_Unicode as a string.
 **/

 public String stringize() {
  return unicode;
  }
 }
