//
// ConstantUtf8.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * A container for a CONSTANT_Utf8 record in a Java <tt>.class</tt> file.
 *
 * @version 1.1, 01 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John Leach</a>
 * @see ConstantFactory
 * @see ConstantUnicode
 * @see ConstantInteger
 * @see ConstantFloat
 * @see ConstantLong
 * @see ConstantDouble
 * @see ConstantClass
 * @see ConstantString
 * @see ConstantFieldRef
 * @see ConstantMethodRef
 * @see ConstantInterfaceMethodRef
 * @see ConstantNameAndType
**/

public final class ConstantUtf8 extends Constant {

 /** The record as a String. */
 String utf8;

 /** The exception error message. */
 private static final String error = "Utf8 record corrupt: ";

 /**
  * Creates a CONSTANT_Utf8 record from a byte array.
  * @param buffer     the byte array
  * @param index      the offset in the input buffer
  * @exception ClassException	    if the record cannot be decoded
 **/

 public ConstantUtf8(byte[] buffer, int index) throws ClassException {
  try {
   size = ValueDecoder.readU2(buffer, index);
   index += 2;

   char unicode[] = new char[size];
   int length = size;
   int unicodeLength = 0;
   int code, finalChar;

   while(length > 0) {
    finalChar = 0;
    code = buffer[index] & 0xFF;
    ++index;
    --length;

    if(code >= 0x80) {
     if((code & 0xE0) == 0xC0) {
      if(length <= 0) throw new ClassException(error + index);
      finalChar = ((code & 0x1F) << 6);
      code = buffer[index] & 0xFF;
      ++index;
      --length;
      if((code & 0xC0) != 0x80) throw new ClassException(error + index);
      finalChar += (code & 0x3F);
      }
     else if((code & 0xF0) == 0xE0) {
      if(length <= 1) throw new ClassException(error + index);
      finalChar = ((code & 0x0F) << 12);
      code = buffer[index] & 0xFF;
      ++index;
      --length;
      if((code & 0xC0) != 0x80) throw new ClassException(error + index);
      finalChar += ((code & 0x3F) << 6);
      code = buffer[index] & 0xFF;
      ++index;
      --length;
      if((code & 0xC0) != 0x80) throw new ClassException(error + index);
      finalChar += (code & 0x3F);
      }
     else throw new ClassException(error + index);
     }
    else finalChar = code;

    unicode[unicodeLength] = (char)finalChar;
    ++unicodeLength;
    }
   utf8 = new String(unicode, 0, unicodeLength);

   // update the size and type

   size += (2 + 1);
   type = Constant.UTF8;
   }
  catch(ArrayIndexOutOfBoundsException e) {
   throw new ClassException(error + index);
   }
  }

 /**
  * Gets the CONSTANT_Utf8 as a string.
 **/

 public String stringize() {
  return utf8;
  }
 }
