//
// ExceptionElement.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * Defines an ExceptionElement within a Code attribute record in a Java
 * <tt>.class</tt> file
 *
 * @version 1.0, 02 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John Leach</a>
 * @see CodeAttribute
**/

public final class ExceptionElement {

 /** The element size. */
 static final int size = 8;

 /** The start program counter (inclusive). */
 int startProgramCounter;

 /** The end program counter (exclusive). */
 int endProgramCounter;

 /** The handler program counter. */
 int handlerProgramCounter;

 /** The catch object type. */
 int catchType;

 /** The catch object name. */
 String catchName;

 /** The exception error message. */
 private static final String error = "ExceptionElement record corrupt: ";

 /**
  * Creates an ExceptionElement from a byte array.
  * @param buffer     the byte array
  * @param index      the offset in the input buffer
  * @exception ClassException	if the record cannot be decoded
 **/

 public ExceptionElement(byte[] buffer, int index) throws ClassException {
  try {
   startProgramCounter = ValueDecoder.readU2(buffer, index);
   index += 2;

   endProgramCounter = ValueDecoder.readU2(buffer, index);
   index += 2;

   handlerProgramCounter = ValueDecoder.readU2(buffer, index);
   index += 2;

   catchType = ValueDecoder.readU2(buffer, index);
   index += 2;

   try {
    if(catchType == 0) catchName = "Ljava/lang/Object;";
    else {
     if(Constant.constantPool[catchType].type != Constant.CLASS) throw new ClassException(error + index + " [wrong type for catch]");
     ConstantClass classDescriptor = (ConstantClass)Constant.constantPool[catchType];
     catchName = classDescriptor.getSignature().sig;
     }
    }
   catch(NullPointerException e) {
    throw new ClassException(error + index + " [no entry for catch]");
    }
   catch(ArrayIndexOutOfBoundsException e) {
    throw new ClassException(error + index + " [no entry for catch]");
    }
   }
  catch(ArrayIndexOutOfBoundsException e) {
   throw new ClassException(error + index);
   }
  }
 }
