//
// LineNumberElement.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * Defines a LineNumber element within a LineNumberTable attribute record in a
 * Java <tt>.class</tt> file
 *
 * @version 1.0, 02 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John Leach</a>
 * @see LineNumberTableAttribute
**/

public final class LineNumberElement {

 /** The element size. */
 static final int size = 4;

 /** The program counter. */
 int programCounter;

 /** The line number. */
 int lineNumber;

 /** The exception error message. */
 private static final String error = "LineNumberElement record corrupt: ";

 /**
  * Creates a LineNumberElement from a byte array.
  * @param buffer     the byte array
  * @param index      the offset in the input buffer
  * @exception ClassException	if the record cannot be decoded
 **/

 public LineNumberElement(byte[] buffer, int index) throws ClassException {
  try {
   programCounter = ValueDecoder.readU2(buffer, index);
   index += 2;

   lineNumber = ValueDecoder.readU2(buffer, index);
   index += 2;
   }
  catch(ArrayIndexOutOfBoundsException e) {
   throw new ClassException(error + index);
   }
  }
 }
