//
// LocalVariableElement.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * Defines a LocalVariable element within a LocalVariableTable attribute
 * record in a Java <tt>.class</tt> file
 *
 * @version 1.0, 02 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John Leach</a>
 * @see LocalVariableTableAttribute
**/

public final class LocalVariableElement {

 /** The element size. */
 static final int size = 10;

 /** The start program counter. */
 int startPC;

 /** The end program counter (exclusive). */
 int endPC;

 /** The slot number. */
 int slotNumber;

 /** The element name. */
 int nameIndex;

 /** The signature. */
 int signatureIndex;

 /** The exception error message. */
 private static final String error = "LocalVariableElement record corrupt: ";

 /**
  * Creates a LocalVariableElement from a byte array.
  * @param buffer     the byte array
  * @param index      the offset in the input buffer
  * @exception ClassException	if the record cannot be decoded
 **/

 public LocalVariableElement(byte[] buffer, int index) throws ClassException {
  try {
   startPC = ValueDecoder.readU2(buffer, index);
   index += 2;

   endPC = startPC + ValueDecoder.readU2(buffer, index);
   index += 2;

   nameIndex = ValueDecoder.readU2(buffer, index);
   index += 2;

   signatureIndex = ValueDecoder.readU2(buffer, index);
   index += 2;

   slotNumber = ValueDecoder.readU2(buffer, index);
   }
  catch(ArrayIndexOutOfBoundsException e) {
   throw new ClassException(error + index);
   }
  }
 }
