//
// SourceFileAttribute.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * Decodes a SourceFile attribute record in a Java <tt>.class</tt> file
 *
 * @version 1.0, 02 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John Leach</a>
 * @see Attribute
 * @see AttributeFactory
 * @see ConstantValueAttribute
 * @see CodeAttribute
 * @see ExceptionsAttribute
 * @see LineNumberTableAttribute
 * @see LocalVariableTableAttribute
 * @see UnknownAttribute
**/

public final class SourceFileAttribute extends Attribute {

 /** The source filename index. */
 int sourceIndex;

 /** The exception error message. */
 private static final String error = "SoureFile attribute record corrupt: ";

 /** The attribute value. */
 private String attributeValue;

 /**
  * Creates a SourceFile attribute record from a byte array.
  * @param buffer     the byte array
  * @param index      the offset in the input buffer
  * @exception ClassException	if the record cannot be decoded
 **/

 public SourceFileAttribute(byte[] buffer, int index) throws ClassException {
  try {
   int attrSize = ValueDecoder.readU4(buffer, index);
   if(attrSize != 2) throw new ClassException(error + index + " [size != 2]");
   index += 4;

   sourceIndex = ValueDecoder.readU2(buffer, index);
   try {
    if(Constant.constantPool[sourceIndex].type == Constant.UTF8) attributeValue = Constant.constantPool[sourceIndex].stringize();
    else throw new ClassException(error + index + " [wrong type for source]");
    }
   catch(NullPointerException e) {
    throw new ClassException(error + index + " [no entry for source]");
    }
   catch(ArrayIndexOutOfBoundsException e) {
    throw new ClassException(error + index + " [no entry for source]");
    }

   // update the size and type

   size = 4 + 2;
   type = Attribute.SOURCE_FILE;
   }
  catch(ArrayIndexOutOfBoundsException e) {
   throw new ClassException(error + index);
   }
  }

 /**
  * Gets the attribute as a string.
 **/

 public String stringize() {
  return attributeValue;
  }
 }
