//
// UnknownAttribute.java
//
// Copyright (c) 1996 John Leach. All Rights Reserved.
//
// Permission to use, copy, modify, and distribute this software and its
// documentation for NON-COMMERCIAL purposes and without fee is hereby granted
// provided that this copyright notice appears in all copies.
//
// JOHN LEACH MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
// THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
// NON-INFRINGEMENT. JOHN LEACH SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
// BY A LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE
// OR ITS DERIVATIVES.
//

package jclass;

/**
 * Decodes an unknown attribute record in a Java <tt>.class</tt> file
 *
 * @version 1.0, 02 September 1996
 * @author  <a href="mailto:leachj@easy1.easynet.it">John Leach</a>
 * @see Attribute
 * @see AttributeFactory
 * @see SourceFileAttribute
 * @see ConstantValueAttribute
 * @see CodeAttribute
 * @see ExceptionsAttribute
 * @see LineNumberTableAttribute
 * @see LocalVariableTableAttribute
**/

public final class UnknownAttribute extends Attribute {

 /** The attribute data. */
 byte data[];

 /** The attribute name. */
 String attributeName;

 /** The exception error message. */
 private static final String error = "Unknown attribute record corrupt: ";

 /**
  * Creates an unknown attribute record from a byte array.
  * @param buffer     the byte array
  * @param index      the offset in the input buffer
  * @param name       the unknown attribute name
  * @exception ClassException	if the record cannot be decoded
 **/

 public UnknownAttribute(byte[] buffer, int index, String name) throws ClassException {
  try {

   // NOTE: the size is an unsigned 32 bit value in the specs.

   size = ValueDecoder.readU4(buffer, index);
   index += 4;

   if(size > 0) {
    data = new byte[size];
    System.arraycopy(buffer, index, data, 0, size);
    }

   // update the size and type

   size += 4;
   type = Attribute.UNKNOWN;

   // store the name

   attributeName = name;
   }
  catch(ArrayIndexOutOfBoundsException e) {
   throw new ClassException(error + index);
   }
  }

 /**
  * Gets the attribute as a string.
 **/

 public String stringize() {
  return "Unknown attribute record: " + attributeName;
  }
 }
